import click
import os
from pathlib import Path
from colorama import Fore, Style
import shutil
import toml

def update_pipeline_order(pipeline_name):
    """
    Update the PIPELINE_ORDER list in the main.py file with the given pipeline name.

    Args:
        pipeline_name (str): The name of the pipeline to be added to the PIPELINE_ORDER list.
    """
    current_directory = os.getcwd()
    main_py_path = os.path.join(current_directory, 'main.py')
    print(main_py_path)
    if not os.path.exists(main_py_path):
        click.echo(f"{Fore.RED}main.py file not found in the current directory.{Style.RESET_ALL}")
        return

    pipeline_found = False  # Variable to track if PIPELINE_ORDER was found

    try:
        with open(main_py_path, 'r') as f:
            lines = f.readlines()

        for i, line in enumerate(lines):

            if 'PIPELINE_ORDER' in line:
                start_index = line.find('[')
                end_index = line.rfind(']')
                pipeline_order = line[start_index+1:end_index]
                pipeline_order = [element.strip() for element in pipeline_order.split(',') if element.strip()]
                pipeline_order.append(f'"{pipeline_name}"')

                for j in range(len(pipeline_order) - 1):
                    if not pipeline_order[j].endswith('"') and not pipeline_order[j+1].startswith('"'):
                        pipeline_order[j] += ','

                new_line = f'        PIPELINE_ORDER = [{", ".join(pipeline_order)}]\n'
                lines[i] = new_line
                pipeline_found = True  # Set to True when PIPELINE_ORDER is found
                break

        # Check if PIPELINE_ORDER was not found
        if not pipeline_found:
            raise ValueError('PIPELINE_ORDER must be set in main.py and a valid list with "" separation')

        with open(main_py_path, 'w') as f:
            f.writelines(lines)

        click.echo(f"{Fore.GREEN}PIPELINE_ORDER updated successfully.{Style.RESET_ALL}")

    except Exception as e:
        click.echo(f"{Fore.RED}Error updating PIPELINE_ORDER: {e}{Style.RESET_ALL}")


@click.command()
@click.option('--pipeline-name', prompt=f'{Fore.YELLOW}Please enter the pipeline name{Style.RESET_ALL}',help='Name of the project')
def main(pipeline_name):
    """
    Create a new pipeline with the given name.

    Args:
        pipeline_name (str): The name of the pipeline to be created.
    """
    current_directory = os.getcwd()
    toml_path = os.path.join(current_directory, 'project.toml')
    if not os.path.exists(toml_path):
        click.echo(f"{Fore.RED}project.toml file not found in the current directory.{Style.RESET_ALL}")
        return

    with open(toml_path, 'r') as f:
        project_config = toml.load(f)
        pipeline_folder = project_config['project']['place']

    pipeline_folder = os.path.join(current_directory, pipeline_folder, pipeline_name)
    if os.path.exists(pipeline_folder):
        click.echo(f"{Fore.RED}Pipeline '{pipeline_name}' already exists.{Style.RESET_ALL}")
        return

    os.makedirs(pipeline_folder)
    init_file = os.path.join(pipeline_folder, "__init__.py")
    Path(init_file).touch()

    templates_folder = os.path.join(os.path.dirname(__file__), '..', 'templates')

    # Copy pipeline.py from the template folder to the pipeline folder
    pipeline_py_path = os.path.join(pipeline_folder, 'pipeline.py')
    shutil.copy(os.path.join(templates_folder, 'pipeline.py'), pipeline_py_path)

    # Replace __PIPE__ with the actual pipeline name in pipeline.py
    with open(pipeline_py_path, 'r') as f:
        pipeline_content = f.read()

    new_content = pipeline_content.replace('__PIPE__', pipeline_name)

    with open(pipeline_py_path, 'w') as f:
        f.write(new_content)

    # Copy nodes.py from the template folder to the pipeline folder
    shutil.copy(os.path.join(templates_folder, 'nodes.py'), pipeline_folder)

    update_pipeline_order(pipeline_name)

    click.echo(f"{Fore.GREEN}New pipeline '{pipeline_name}' created successfully.{Style.RESET_ALL}")


if __name__ == '__main__':
    main()



import importlib.util
from pathlib import Path
import toml
from .node import Node
import inspect
from jetline.data.data import DataManager
from jetline.logging import logger
import os, sys

class Pipeline:
    def __init__(self):
        """
         Initialize the nodes dictionary. This is called by __init__ to initialize the node dictionary and its sub - classes
        """
        self.nodes = {}
      

    def run(self):
        """
         Run the nodes and return outputs. This is the entry point for the Node class. You can override this method to do something different
         
         
         Returns: 
             A dictionary of outputs keyed by
        """
        outputs = {}
        # Execute all nodes in the graph.
        for node_name, node in self.nodes.items():
            outputs[node_name] = node.execute()
        return outputs

    def add_node(self, name, func, inputs):
        """
         Add a node to the graph. This is a convenience method for creating a : class : ` Node ` and associating it with the graph.
         
         Args:
             name: The name of the node. It must be unique among all nodes in the graph.
             func: The function to run on the node. It must take a single argument : an instance of
             inputs: The inputs to the
        """
        self.nodes[name] = Node(name, func, inputs)

    def register(self, pipeline_instance):
        """
         Register a pipeline with the pipeline manager. This is a convenience method for the use of : meth : ` Pipeline. nodes `
         
         Args:
             pipeline_instance: An instance of : class : ` Pipeline
        """
        for pipline in pipeline_instance:
            if pipline not in self.nodes:
                raise ValueError(f"Pipeline '{pipline}' ist erforderlich, wurde aber nicht gefunden.")
        self.nodes.update(pipeline_instance.nodes)


class PipelineManager:
    def __init__(self):
        """
        Initialize the pipeline manager.
        """
        self.caller_path = self.get_caller_path()
        self.add_project_directory_to_sys_path()
        self.data_manager = DataManager(caller_path=self.caller_path)
        self.pipelines = {}
        self.load_pipelines()
       

    def get_caller_path(self):
        """
        Ermittelt den Pfad des Skripts, das diesen DataManager instanziiert.
        """
        frame = inspect.stack()[2]
        module = inspect.getmodule(frame[0])
        if module is not None:
            module_path = Path(module.__file__).parent
        else:
            module_path = Path.cwd()  
        return module_path
    
    def add_project_directory_to_sys_path(self):
        """Fügt das Projektverzeichnis zum sys.path hinzu."""
       
        toml_file_path = os.path.join(self.caller_path, 'project.toml')

        with open(toml_file_path, 'r') as f:
            toml_data = toml.load(f)
            place = toml_data.get('project', {}).get('place')

        if not place:
            print("Fehler: 'place' nicht gefunden in der project.toml-Datei.")
            sys.exit(1)

        target_directory = os.path.join(self.caller_path, place)

        sys.path.append(target_directory)
    

    def load_pipelines(self):
        """
        Load Pipelines from project.toml and create modules.
        """
        base_path = self.caller_path

        project_toml_path = base_path / "project.toml"
        with open(project_toml_path, "r") as f:
            project_config = toml.load(f)
            pipeline_folder_name = project_config["project"]["place"]
            pipeline_folder = base_path / pipeline_folder_name
        
            for pipeline_name in pipeline_folder.iterdir():
                if pipeline_name.is_dir():
                    pipeline_name = pipeline_name.name
                    pipeline_module_path = pipeline_folder / pipeline_name / "pipeline.py"
                    if pipeline_module_path.exists():
                        spec = importlib.util.spec_from_file_location("pipeline_module", pipeline_module_path)
                        pipeline_module = importlib.util.module_from_spec(spec)
                        spec.loader.exec_module(pipeline_module)
                        pipeline_register_function = getattr(pipeline_module, "register", None)
                        if callable(pipeline_register_function):
                            # Übergeben Sie den data_manager-Parameter an die register-Funktion
                            pipeline_definition = pipeline_register_function(self.data_manager)
                            if isinstance(pipeline_definition, Pipeline):
                                self.pipelines[pipeline_name] = pipeline_definition
                            else:
                                raise ValueError(f"The register function in {pipeline_module_path} must return a Pipeline instance.")

    def run(self, pipeline_order):
        """
        Run a list of pipelines in the order they were added.
        """
        results = {}
        
        missing_pipelines = [pipeline_name for pipeline_name in pipeline_order if pipeline_name not in self.pipelines]
        if missing_pipelines:
            error_message = f"Error running pipelines: The following pipelines were not found: {', '.join(missing_pipelines)}"
            logger.error(error_message)
            raise ValueError(error_message)
        
        for pipeline_name in pipeline_order:
            pipeline = self.pipelines[pipeline_name]
            results[pipeline_name] = pipeline.run()
            logger.info(f"Pipeline '{pipeline_name}' executed successfully.")
        
        logger.info("All Pipelines ran successfully.")
        return results



class Node:
    def __init__(self, name, function, inputs):
        """
         Initializes the Function object. This is the method that should be called by the user to initialize the Function object
         
         Args:
         	 name: The name of the function
         	 function: The function that will be called when the function is called
         	 inputs: The inputs that will be passed to the function
        """
        self.name = name
        self.function = function
        self.inputs = inputs

    def execute(self):
        """
         Execute the function with the inputs. This is the method that will be called by the : class : ` Function ` when it is called.
         
         
         Returns: 
         	 The result of the function ( if any ) or None if there was an error ( in which case the error will be logged
        """
        return self.function(*self.inputs)


from setuptools import setup, find_packages

classifiers = [
    'Development Status :: 3 - Alpha',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: MIT License',
    'Operating System :: OS Independent',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8',
    'Programming Language :: Python :: 3.9',
    'Programming Language :: Python :: 3.10',
    'Topic :: Software Development :: Libraries :: Python Modules'
]

install_requires = [
    'click==8.1.7',
    'colorama==0.4.6',
    'toml==0.10.2',
    'pandas==2.2.0',
    'openpyxl==3.1.2',
    'coloredlogs==15.0.1',
    'fastapi==0.110.0',

]

setup(
    name='jetline',
    version='0.0.43',
    description='Automated Pipeline Builder',
    url='https://github.com/your_username/jetline',
    author='Johannes Kanthak',
    author_email='johannes.kanthak@kdc-solutions.de',
    license='MIT',
    classifiers=classifiers,
    keywords='pipeline automation',
    packages=find_packages(),
    include_package_data=True,
    install_requires=install_requires,
    entry_points={
        'console_scripts': [
            'jetline-setup = jetline.commands.installer:main',
            'jetline-info = jetline.commands.info:main',
            'jetline-create-pipe = jetline.commands.create_pipe:main',
            'jetline-analyze-project = jetline.commands.analyze_project:main',
        ],
    },
)




def node_function_1(data, multiplier, data_manager) -> int:
   
    some_data = data_manager.get_jetline_data("Name")
    
    result = data * multiplier * some_data
    return result


def node_function_2(data, offset) -> int:
   
   
    result = data + offset
    return result
